/* 
   CWrap 0.1.6: linux console wrapster utility
   Author     : Scott Petersen [peterses@oit.edu]
   Date       : 9.22.00

   Encodes/Decodes wrapster archives.  

   And, if anyone knows how to calculate the
   CRC and cares to explain it, send me an email ;)
*/

#include <stdio.h>
#include <fcntl.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>

void print_help (char *argv0);
void make_archive (int argc, char **argv);
void extract_archive (int argc, char **argv);
char *strip_path (char *file);

const char header[4]  = { 0xFF, 0xFB, 0x18, 0x0C };
const char *signature = "wrapster\0\0";
const char *version   = "2.0\0\0\0\0\0\0";

int main(int argc, char **argv)
{
        fprintf(stdout, ",- CWrap 0.1.6: linux console wrapster utility\n"
                        "|- Author: Scott Petersen [peterses@oit.edu]\n|\n");

	if (argc < 3)
		print_help (argv[0]);

	if (!strcmp(argv[1], "-c"))					/* compress files */
		make_archive (argc, argv);
	else if (!strcmp(argv[1], "-d"))				/* decompress wrapster archive */
		extract_archive (argc, argv);
	else								/* wrong options, display syntax */
		print_help (argv[0]);

	return 0;
}

void print_help (char *argv0)
{
	fprintf (stderr, "|- Syntax: %s <-c|-d> <file(s)> [output mp3]\n", argv0);
	fprintf (stderr, "`- Note: output file is only for compression\n");
	
	exit (1);
}

void extract_archive (int argc, char **argv)
{
	int infile, outfile, filecount, i = 0, filesize;
	char *buffer;
	char *filename;

        infile = open (argv[2], O_RDONLY);
        if (infile == -1)
        {
	        fprintf(stderr, "`- Error opening file: %s\n", argv[2]);
                exit (1);
        }

        buffer = malloc(256 * sizeof(char));
        memset (buffer, 0, sizeof(buffer));

        read (infile, buffer, 4);                                       /* header */
        fprintf (stdout, "|- Header   : %#hx %#hx %#hx %#hx\n", buffer[0], buffer[1], buffer[2], buffer[3]);
        read (infile, buffer, 6);                                       /* reserved */
        read (infile, buffer, 10);                                      /* signature */
        fprintf (stdout, "|- Signature: %s\n", buffer);
        memset (buffer, 0, sizeof(buffer));
        read (infile, buffer, 8);                                       /* version */
        fprintf (stdout, "|- Version  : %s\n", buffer);
        read (infile, &filecount, 4);                                   /* filecount */
        fprintf (stdout, "|- Filecount: %d\n|\n", filecount);

        free (buffer);

        for (i = 0; i < filecount; i++)
        {
	        buffer = malloc (256 * sizeof(char));
                memset (buffer, 0, sizeof(buffer));
                read (infile, buffer, 256);                             /* filename */
		filename = strip_path(buffer);
                fprintf (stdout, "|- Filename : %-50.50s (", filename);
                outfile = open (filename, O_EXCL | O_WRONLY | O_CREAT, S_IRUSR | S_IWUSR);
		free(filename);
                if (outfile == -1)
                {
	                fprintf(stdout, "      file exists)\n");
			read (infile, buffer, 36);
			read (infile, &filesize, 4);
			free(buffer);
			buffer = malloc(filesize * sizeof(char));
			read (infile, buffer, filesize);
                      	free (buffer);
                      	continue;
                }

                memset (buffer, 0, sizeof(buffer));
                read (infile, buffer, 32);                              /* reserved */
                read (infile, buffer, 4);                               /* CRC32 */
                read (infile, &filesize, 4);                            /* filesize */
                fprintf (stdout, "%11d bytes)\n", filesize);

                free (buffer);

                buffer = malloc (filesize * sizeof(char));
                read (infile, buffer, filesize);                        /* data */
                write (outfile, buffer, filesize);

                free (buffer);
                close (outfile);
	}

        fprintf (stdout, "|\n`- Finished Extracting.\n");
        close(infile);
}

void make_archive (int argc, char **argv)
{
        int infile, outfile, filecount = 0, i = 0;
        char *buffer, destination[256], command[256];
        struct stat ifstat;
	char *filename;

        if (argc < 4)
	        print_help (argv[0]);

        strcpy(destination, argv[argc - 1]);

        outfile = open (destination, O_WRONLY | O_CREAT | O_TRUNC, S_IRWXU);
        if (outfile == -1)
        {
                fprintf (stderr, "Error opening file for writing: %s\n", argv[2]);
                exit (1);
        }

        /* count the files, discarding any that match this program or ./<this program> */
        for (i = 2; i < argc - 1; i++)
        {
                sprintf(command, "./%s", argv[i]);
                if (strcmp(argv[0], argv[i]) && strcmp(argv[0], command))
	                filecount++;
        }

        buffer = malloc(256 * sizeof(char));
        memset (buffer, 0, sizeof (buffer));

        write (outfile, header, 4);                                     /* header */
        fprintf (stdout, "|- Wrote header   : %#hx %#hx %#hx %#hx\n", header[0], header[1], header[2], header[3]);
        write (outfile, buffer, 6);                                     /* reserved data */
        write (outfile, signature, 10);                                 /* signature */
        fprintf (stdout, "|- Wrote signature: %s\n", signature);
        write (outfile, version, 8);                                    /* version */
        fprintf (stdout, "|- Wrote version  : %s\n", version);
        write (outfile, &filecount, 4);                                 /* filecount */
        fprintf (stdout, "|- Wrote filecount: %d\n|\n", filecount);

        for (i = 2; i < argc - 1; i++)
        {
	        memset (command, 0, sizeof(command));
                sprintf(command, "./%s", argv[i]);
                if (!strcmp (argv[0], argv[i]) || !strcmp(argv[0], command))
                        continue;

                infile = open (argv[i], O_RDONLY);
                if (infile == -1)
                {
                        fprintf (stderr, "`- Error opening file for input: %s\n", argv[i]);
                        close (outfile);
                        unlink (destination);
                        exit (1);
                }

                buffer = malloc(256 * sizeof(char));
		filename = strip_path(argv[i]);
                write (outfile, filename, 256);                           /* filename */

                memset (buffer, 0, sizeof(buffer));
                write (outfile, buffer, 32);                            /* reserved data */
                write (outfile, buffer, 4);                             /* CRC32 */

                fstat (infile, &ifstat);
                write (outfile, &ifstat.st_size, 4);                    /* file size */

                free(buffer);

                buffer = malloc(ifstat.st_size);
                memset (buffer, 0, sizeof(buffer));
                read (infile, buffer, ifstat.st_size);
                write (outfile, buffer, ifstat.st_size);                /* file data */

                fprintf(stdout, "|- Added File     : %s\n", filename);

		free (filename);
                free (buffer);
                close (infile);
	}

        buffer = malloc(4096 * sizeof(char));
        memset (buffer, 0, sizeof(buffer));
        write (outfile, buffer, 4096);                                  /* padding */

        fprintf (stdout, "|\n`- Written        : %s\n", destination);
        free (buffer);
        close (outfile);
}

char *strip_path(char *file)
{
        char *new_string;
        int i = 0, j = 0;

	new_string = malloc(256 * sizeof(char));
        memset(new_string, 0, strlen(new_string));

        for (i = 0; i< strlen(file); i++)
        {
                if (file[i] == '/')
                {
                        memset(new_string, 0, strlen(new_string));
                        j = 0;
                }
                else
                {
                        new_string[j++] = file[i];
                }
        }
	new_string[j] = '\0';
        return new_string;
}
